import os.path
import matplotlib.pyplot as plt
import torch
import filetype
import cv2
import numpy as np
import tifffile as tif
import sklearn.decomposition as sd
from scipy.signal import medfilt2d
from player import player, save_tiff,save_avi


def plot_frame(frame):
    fig, ax = plt.subplots(1, 2)
    flatten_frame = torch.flatten(frame)
    data_index = range(len(flatten_frame))
    ax[0].imshow(frame)
    ax[0].axis('off')
    ax[1].scatter(data_index, flatten_frame)
    ax[1].set_ylabel('Data Intensity')
    plt.show()


def frame_z_score(x):
    std, mean = torch.std_mean(x, dim=(1, 2), keepdim=True)
    return (x - mean) / std


def read(data_path):
    data_path = os.path.join(os.path.dirname(__file__), os.pardir, data_path)
    if filetype.guess(data_path).extension == 'avi':
        cap = cv2.VideoCapture(data_path)
        frame_width = int(cap.get(3))
        frame_height = int(cap.get(4))
        frame_number = int(cap.get(7))
        frame = np.zeros((frame_number, frame_height, frame_width), dtype=np.single)
        count = 0
        ret = True
        while ret:
            ret, img = cap.read()
            if ret:
                img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
                frame[count] = img
                count += 1
        image_array = torch.tensor(frame)
        if len(image_array.shape) == 3:
            image_array = torch.unsqueeze(image_array, -1)
    elif filetype.guess(data_path).extension == 'tif':
            image_array = torch.tensor(tif.imread(data_path).astype(float))  # image shape (THWC)
            if len(image_array.shape) == 3:
                image_array = torch.unsqueeze(image_array, -1)
    return image_array

def pre_pca(data_path):
    # Z-score preprocessing
    image_array = read(data_path)
    image_array_center = frame_z_score(image_array)
    #save_tiff(image_array, name='raw')
    return image_array_center

'''
###################     new pca     ######################
# Remove the first Principal Component in time dimension to clean background
def bgclean_pca_filter(data_path):
    image_array = pre_pca(data_path)
    image_array_pca = image_array.view(image_array.shape[0], -1).T
    n_comp = int((image_array.shape[0])*0.75)
    pca_2 = PCA(n_components=100)
    #tryt = pca_2.fit(image_array_pca)
    reduced_x2 = pca_2.fit_transform(image_array_pca) #fit_transform（X[，y]）将模型与X配合，并在X上应用降维。
   
    explained_var_ratio = pca_2.explained_variance_ratio_
    # 计算累计方差比例，以确定前100个主成分的方差总和
    cumulative_var_ratio = np.cumsum((explained_var_ratio),axis=0)
    filtered_digits_2 = pca_2.inverse_transform(reduced_x2)
    filtered_digits_keep = filtered_digits_all[:,1:100]
    after_pca_all = image_array_pca - filtered_digits
    after_pca[after_pca < 0] = 0
    after_pca = torch.reshape(after_pca.T, image_array.shape)
    return after_pca

X_pca_deleted = X_pca[:, 1:101]
'''

##################  old pca   ###########################
def bgclean_pca_filter(data_path,pca_number):
    image_array = pre_pca(data_path)
    image_array_pca = image_array.view(image_array.shape[0], -1).T
    pca1 =sd.PCA(n_components=pca_number)
    reduced_x1 = pca1.fit_transform(image_array_pca)
    filtered_digits1 = torch.tensor(pca1.inverse_transform(reduced_x1))
    tryt=torch.reshape(filtered_digits1.T, image_array.shape)
    tryt[tryt < 0] = 0
    #save_avi(tryt,'Daria3CP')
    #print("Variance of 10CPS : %.3f " %(sum(pca1.explained_variance_ratio_)))
    after_pca1 = image_array_pca - filtered_digits1
    after_pca1[after_pca1 < 0] = 0
    after_pca1 = torch.reshape(after_pca1.T, image_array.shape)
    #save_avi(after_pca1,"tttt")
    return after_pca1


# Remove the redundant Principal Component in time dimension
def time_reduction_pca_filter(data_path, n_components, spatial_size):
    image_array = pre_pca(data_path)
    image_array_pca = image_array.view(image_array.shape[0], -1).T
    pca = sd.PCA(n_components=n_components)
    reduced_x = pca.fit_transform(image_array_pca)
    x_rnf = np.zeros_like(reduced_x)
    for i in range(n_components):
        x_rnf[:, i] = np.reshape(medfilt2d(np.reshape(reduced_x[:, i], (image_array.shape[1], -1)), kernel_size=spatial_size), (-1,))
    return torch.tensor(x_rnf).T


# For Function file testing
if __name__ == '__main__':
     xx=bgclean_pca_filter("/Users/wenli/Desktop/dataset_ML/vid3_slice1_TTX_CNO_03_03_231hz.avi",5)
     
