# =============================================================================
# ------------------------- video to array
# =============================================================================
import numpy as np
import matplotlib.pyplot as plt
import cv2
import torch
from pca_filter import read, pre_pca
from player import save_tiff, save_avi
from PIL import Image, ImageFilter
import scipy.ndimage.morphology as morphology
import numpy as np
import cv2

def motion_correction_video(input_path,output_path,nmarqueurs,nfeatures,patchSize):
    cap = cv2.VideoCapture(input_path)
    # 获取第一帧作为模板
    ret, template_frame = cap.read()
    template_frame = cv2.cvtColor(template_frame, cv2.COLOR_BGR2GRAY)
    num_frames,frame_width,frame_height = int(cap.get(cv2.CAP_PROP_FRAME_COUNT)),int(cap.get(cv2.CAP_PROP_FRAME_WIDTH)), int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
    # 创建输出视频文件
    fourcc = cv2.VideoWriter_fourcc(*'MJPG')
    out = cv2.VideoWriter(output_path, fourcc, 100.0, (frame_width, frame_height))
    # 创建ORB检测器
    orb_detector = cv2.ORB_create(nfeatures=nmarqueurs,nfeatures=30, patchSize=30)
    # 使用第一帧作为模板进行特征匹配和运动校正
    for _ in range(num_frames - 1):
        ret, target_frame = cap.read()
        if not ret:
            break
        # 将当前帧转换为灰度图像
        target_gray = cv2.cvtColor(target_frame, cv2.COLOR_BGR2GRAY)
        # 检测关键点和描述符
        kp1, d1 = orb_detector.detectAndCompute(target_gray, None)
        kp2, d2 = orb_detector.detectAndCompute(template_frame, None)

        matcher = cv2.BFMatcher(cv2.NORM_HAMMING, crossCheck=True)
        matches = matcher.match(d1, d2)
        matches = sorted(matches, key=lambda x: x.distance)
 # 获取匹配点对应的坐标
        p1 = np.float32([kp1[m.queryIdx].pt for m in matches]).reshape(-1, 1, 2)
        p2 = np.float32([kp2[m.trainIdx].pt for m in matches]).reshape(-1, 1, 2)
 # 计算运动矩阵
        homography, _ = cv2.findHomography(p1, p2, cv2.RANSAC)
        # 进行透视变换
        transformed_frame = cv2.warpPerspective(target_frame, homography, (frame_width, frame_height))
        out.write(transformed_frame)
    # 释放资源
    cap.release()
    out.release()
    cv2.destroyAllWindows()



# 示例用法
input_path = '/Users/wenli/Desktop/saclay/juliette/Souris_6_Video_1_1.avi'
output_path = '/Users/wenli/Desktop/saclay/juliette/Souris_6_Video_1_1afterMC.avi'
zz=motion_correction_video(input_path,output_path)
#xx = video_motion_correction(input_path,output_path)
#save_avi(zz, "afterMC")


