#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jun 27 12:22:30 2023
@author: wenli
"""

import os.path
import datetime
import cv2
import torch
import tifffile
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.animation as animation
import seaborn as sns
import mpl_toolkits.mplot3d as mm
import pandas as pd
from matplotlib import cm

def frame_z_score(x):
    std, mean = torch.std_mean(x, dim=(1, 2), keepdim=True)
    return (x - mean) / std

def player(frame, start=0, end=None):
    if end is None:
        end = frame.shape[0]
    frame = frame_z_score(frame[start:end, :, :, :]).numpy().astype(float)
    frame_list = list(frame)
    fig = plt.figure()
    frames = [[plt.imshow(frame_list[i])] for i in range(len(frame_list))]
    ani = animation.ArtistAnimation(fig, frames)
    plt.show()


def image_average(step,frames):
    mean_frame = []
    for x in range(0,len(frames),step):
        framesaverage1=np.mean(frames[x:x+step, :, :, ],axis=0)
        mean_frame.append(framesaverage1)
    meanframesarray= np.array(mean_frame).astype('uint8')
    meanframesarray1 = [frame_temp for frame_temp in meanframesarray]
    return meanframesarray1
    

    
def normalization(frames):
    subsnorm=[]  
    for i in range(len(frames)):
        image_norm = cv2.normalize(frames[i], None, alpha=0, beta=255, norm_type=cv2.NORM_MINMAX)
        subsnorm.append(image_norm)
    return subsnorm
  

def GMMmask(video, mask_part):
    video = torch.tensor(np.array(normalization(video.squeeze().numpy()))).unsqueeze(-1)
    enhance_part = torch.tensor(np.array(normalization(mask_part.squeeze().numpy()))).unsqueeze(-1)*1.5
    enhanced_video = torch.cat((mask_part, video,  mask_part), dim=3)
    return enhanced_video

#################################################
def save_avi(frame, name=None, start=0, end=None, fps=100, enhance=False):
    frame[frame < 0] = 0
    if end is None:
        end = frame.shape[0]
    frame = frame[start:end, :, :]
    if name is None:
        date_creation = datetime.datetime.now().strftime('%Y-%m-%dT%H-%M-%S-%f')
        name = 'video_check'
        name += date_creation
    name += '.avi'
    if end is None:
        end = frame.shape[0]
    #frame_minmax = normalization(frame.reshape(50,502,512).numpy())
    frame_minmax = normalization(frame.squeeze().numpy())
    frame_new = np.uint8(frame_minmax)
    save_path = os.path.join(os.path.dirname(__file__), 'Video_Check')
    os.makedirs(save_path, exist_ok=True)
    save_path = os.path.join(save_path, name)
    save = cv2.VideoWriter(save_path, cv2.VideoWriter_fourcc(*'MJPG'), fps, (frame.shape[2], frame.shape[1]))
    for i in range(frame_new.shape[0]):
        if not enhance:
            img = cv2.cvtColor(frame_new[i], cv2.COLOR_GRAY2RGB)
        else:
            img = frame_new[i]
        save.write(img)
    save.release()

    
def save_tiff(frame, name=None, start=0, end=None):
    if name is None:
        date_creation = datetime.datetime.now().strftime('%Y-%m-%dT%H-%M-%S-%f')
        name = 'video_check'
        name += date_creation
    name += '.tif'
    if end is None:
        end = frame.shape[0]
    save_path = os.path.join(os.path.dirname(__file__), 'Video_Check')
    os.makedirs(save_path, exist_ok=True)
    save_path = os.path.join(save_path, name)
    frame = frame[None, :, :, :, :]
    frame_save_form = frame.permute(1, 0, 4, 2, 3)
    if torch.max(frame) > 500:
        frame_save_form = frame_save_form[start:end, :, :, :, :].to(torch.int32).numpy().astype(np.ushort)
    else:
        frame_save_form = frame_save_form[start:end, :, :, :, :].to(torch.float32).numpy()
    tifffile.imwrite(save_path, frame_save_form, imagej=True, photometric='minisblack', metadata={'axes': 'TZCYX'})



def psnr_compare(x,y, depth): # depth=255 if Gray scale
    x, y = x.numpy(), y.numpy()
    mse = ((x-y)**2).mean()
    psnr = 20 * np.log10(depth/np.sqrt(mse))
    return psnr


def deltaF_F(target_video,baseline):
    pretarget = np.array(normalization(target_video.numpy().squeeze()))
    target_F = pretarget.astype('uint8')
    baseline1 = np.array(normalization(baseline.numpy().squeeze()))
    condition = np.greater_equal(target_F,1)
    times_detector = np.where(condition,1,0)# if have value, pixel = 1
    #if_zero = np.zeros_like(target_F)
    deltaf_f = np.zeros_like(baseline1)
    for x in range(target_F.shape[1]):
        for y in range(target_F.shape[2]):
            for t in range(1, target_F.shape[0]):
                if times_detector[t,x,y] == 1 and times_detector[t-1,x,y] == 0:
                    time_ref = t-1
                    if baseline1[time_ref,x,y] != 0:
                        deltaf_f[t,x,y] = (baseline1[t,x,y] - baseline1[time_ref,x,y])/baseline1[time_ref,x,y]
                if times_detector[t,x,y] == 1 and times_detector[t-1,x,y] == 1:
                    ref1_t = time_ref
                    if baseline1[ref1_t,x,y] != 0:
                        deltaf_f[t,x,y] = (baseline1[t,x,y] - baseline1[ref1_t,x,y])/baseline1[ref1_t,x,y]
    return torch.tensor(deltaf_f)
                   
################################## 2D heatmap##################################
def plot_2d_heatmap(video, targetframe,savename):
    if not isinstance(targetframe, int):
        print("Please enter an integer")
        return
    frames = video.numpy().squeeze()
    onepage = sns.heatmap(frames[targetframe], cmap="RdBu_r", vmin=None, vmax=None, xticklabels=False, yticklabels=False)
    page = plt.savefig(savename)

################################## 3D heatmap##################################

def plot_3d_heatmap(video, targetframe,savename):
    if not isinstance(targetframe, int):
        print("Please enter an integer")
        return
    frames = video.numpy().squeeze()
    m,n = frames[targetframe].shape
    R,C = np.mgrid[:m,:n]
    out = np.column_stack((C.ravel(),R.ravel(), frames[targetframe].ravel()))
    df = pd.DataFrame(out, columns = ['Column_A','Column_B','Column_C'])
    x = df["Column_A"].tolist()
    y = df["Column_B"].tolist()
    z = df["Column_C"].tolist()
    fig = plt.figure(figsize=(10, 10))
    ax = mm.Axes3D(fig)
    surf = ax.plot_trisurf(x, y, z, cmap=cm.jet, linewidth=1)
    plt.savefig(savename)
    fig.colorbar(surf, shrink=0.5, aspect=5)




if __name__ == '__main__':
    a = torch.rand((500, 128, 128, 3))
    player(a, 0, 500)
