#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jun 27 12:19:45 2023
@author: wenli
"""

import sys
#ys.path.append('Input_pipeline')
from pca_filter import  read, pre_pca, bgclean_pca_filter 
import numpy as np
import torch
from matplotlib import pyplot as plt
import scipy.signal as ss 
from player import player, save_avi
from brightness_correction import frame_correction, gaussianblur
import cv2
from matplotlib.ticker import LinearLocator, FormatStrFormatter

def image_average(step,frames):
    mean_frame = []
    for x in range(0,len(frames),step):
        framesaverage1=np.mean(frames[x:x+step, :, :, ],axis=0)
        mean_frame.append(framesaverage1)
    meanframesarray= np.array(mean_frame).astype('uint8')
    meanframesarray1 = [frame_temp for frame_temp in meanframesarray]
    return meanframesarray1
    
def normalization(frames):
    subsnorm=[]  
    for i in range(len(frames)):
        image_norm = cv2.normalize(frames[i], None, alpha=0, beta=255, norm_type=cv2.NORM_MINMAX)
        subsnorm.append(image_norm)
    return subsnorm


def lowpass_filter(sig, cutoff, fs, order=14):
    nyq = 0.5 * fs
    normal_cutoff = cutoff / nyq
    sos = ss.butter(order, normal_cutoff, btype='low', analog=False, output='sos')
    filtered = ss.sosfiltfilt(sos, sig, axis=0)
    return torch.tensor(filtered.copy())


def gaussianblur(image_array, radius):
    image_gaussian = np.zeros_like(image_array.numpy()).squeeze()
    for frame in range(image_array.shape[0]):
        image_gaussian[frame] = cv2.GaussianBlur(image_array[frame].numpy(), (radius, radius), 0)
    return torch.from_numpy(image_gaussian).unsqueeze(-1)


def GMM__model(video, distance):
    lenth=len(video)
    background_subtractor = cv2.createBackgroundSubtractorMOG2(varThreshold=distance)
    #background_subtractor.setHistory(lenth)
    for i in range(video.shape[0]):
        frame = video[i].squeeze().numpy()
        foreground_mask = background_subtractor.apply(frame)
        kernel = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (3, 3))
        foreground_mask = cv2.morphologyEx(foreground_mask, cv2.MORPH_OPEN, kernel)
  
        foreground = cv2.bitwise_and(frame, frame, mask=foreground_mask)
        foreground[foreground < 0] = 0
        if len(video.shape) == 3:
            video[i] = torch.from_numpy(foreground)
        else:
            video[i] = torch.from_numpy(foreground).unsqueeze(-1)
    return video


def GMMmask(video, enhance_part):
    video = torch.tensor(np.array(normalization(video.squeeze().numpy()))).unsqueeze(-1)
    enhance_part = torch.tensor(np.array(normalization(enhance_part.squeeze().numpy()))).unsqueeze(-1)*1.5
    enhanced_video = torch.cat((enhance_part, video, enhance_part), dim=3)
    return enhanced_video

